@@ -1892,6 +1892,31 @@ int pri_transfer_rsp(struct pri *ctrl, q931_call *call, int invoke_id, int is_su
  */
 void pri_aoc_events_enable(struct pri *ctrl, int enable);
 
+enum pri_layer2_persistence {
+	PRI_L2_PERSISTENCE_DEFAULT,
+	/*! Immediately bring layer 2 back up if the peer brings layer 2 down. */
+	PRI_L2_PERSISTENCE_KEEP_UP,
+	/*! Leave layer 2 down if the peer brings layer 2 down. */
+	PRI_L2_PERSISTENCE_LEAVE_DOWN,
+#if 0	/* Possible future option.  Would need to define how long to idle before dropping. */
+	/*! Drop layer 2 on D channel idle. */
+	PRI_L2_PERSISTENCE_IDLE_DROP,
+#endif
+};
+
+/*!
+ * \brief Set the layer2 persistence option.
+ *
+ * \param ctrl D channel controller.
+ * \param option Layer 2 persistence to apply.
+ *
+ * \note
+ * Not all values are supported by all modes.
+ *
+ * \return Nothing
+ */
+void pri_persistent_layer2_option(struct pri *ctrl, enum pri_layer2_persistence option);
+
 #define PRI_DISPLAY_OPTION_BLOCK		(1 << 0)	/*!< Do not pass display text. */
 #define PRI_DISPLAY_OPTION_NAME_INITIAL	(1 << 1)	/*!< Use display in SETUP/CONNECT for name. */
 #define PRI_DISPLAY_OPTION_NAME_UPDATE	(1 << 2)	/*!< Use display in FACILITY/NOTIFY for COLP name if appropriate. */
@@ -2154,7 +2179,7 @@ enum PRI_TIMERS_AND_COUNTERS {
 	PRI_TIMER_T310,	/*!< Maximum time between receiving a CALL_PROCEEDING and receiving a ALERT/CONNECT/DISCONNECT/PROGRESS */
 	PRI_TIMER_T313,	/*!< Wait for CONNECT acknowledge, CPE side only */
 	PRI_TIMER_T314,
-	PRI_TIMER_T316,	/*!< Maximum time between transmitting a RESTART and receiving a RESTART ACK */
+	PRI_TIMER_T316,	/*!< Time to wait for a RESTART ACK before retransmitting RESTART. (Timer enabled if greater than zero.) */
 	PRI_TIMER_T317,
 	PRI_TIMER_T318,
 	PRI_TIMER_T319,
@@ -2196,6 +2221,7 @@ enum PRI_TIMERS_AND_COUNTERS {
 	PRI_TIMER_QSIG_CC_T4,	/*!< Path reservation supervision timeout. */
 
 	PRI_TIMER_T312,			/*!< Supervise broadcast SETUP message call reference retention. */
+	PRI_TIMER_N316,			/*!< Number of times to transmit RESTART before giving up if T316 enabled. */
 
 	/* Must be last in the enum list */
 	PRI_MAX_TIMERS
