@@ -81,6 +81,7 @@ static const struct pri_timer_table pri_timer[] = {
 	{ "T313",           PRI_TIMER_T313,             PRI_ALL_SWITCHES },
 	{ "T314",           PRI_TIMER_T314,             PRI_ALL_SWITCHES },
 	{ "T316",           PRI_TIMER_T316,             PRI_ALL_SWITCHES },
+	{ "N316",           PRI_TIMER_N316,             PRI_ALL_SWITCHES },
 	{ "T317",           PRI_TIMER_T317,             PRI_ALL_SWITCHES },
 	{ "T318",           PRI_TIMER_T318,             PRI_ALL_SWITCHES },
 	{ "T319",           PRI_TIMER_T319,             PRI_ALL_SWITCHES },
@@ -184,6 +185,10 @@ static void pri_default_timers(struct pri *ctrl, int switchtype)
 	ctrl->timers[PRI_TIMER_T309] = 6 * 1000;	/* Time to wait before clearing calls in case of D-channel transient event.  Q.931 specifies 6-90 seconds */
 	ctrl->timers[PRI_TIMER_T312] = (4 + 2) * 1000;/* Supervise broadcast SETUP message call reference retention. T303 + 2 seconds */
 	ctrl->timers[PRI_TIMER_T313] = 4 * 1000;	/* Wait for CONNECT acknowledge, CPE side only */
+#if 0	/* Default disable the T316 timer otherwise the user cannot disable it. */
+	ctrl->timers[PRI_TIMER_T316] = 2 * 60 * 1000;	/* RESTART retransmit timer */
+#endif
+	ctrl->timers[PRI_TIMER_N316] = 2;			/* Send RESTART this many times before giving up. */
 
 	ctrl->timers[PRI_TIMER_TM20] = 2500;		/* Max time awaiting XID response - Q.921 Appendix IV */
 	ctrl->timers[PRI_TIMER_NM20] = 3;			/* Number of XID retransmits - Q.921 Appendix IV */
@@ -290,6 +295,27 @@ static int __pri_write(struct pri *pri, void *buf, int buflen)
 	return res;
 }
 
+/*!
+ * \internal
+ * \brief Determine the default layer 2 persistence option.
+ *
+ * \param ctrl D channel controller.
+ *
+ * \return Default layer 2 persistence option. (legacy behaviour default)
+ */
+static enum pri_layer2_persistence pri_l2_persistence_option_default(struct pri *ctrl)
+{
+	enum pri_layer2_persistence persistence;
+
+	if (PTMP_MODE(ctrl)) {
+		persistence = PRI_L2_PERSISTENCE_LEAVE_DOWN;
+	} else {
+		persistence = PRI_L2_PERSISTENCE_KEEP_UP;
+	}
+
+	return persistence;
+}
+
 /*!
  * \internal
  * \brief Determine the default display text send options.
@@ -558,6 +584,7 @@ static struct pri *pri_ctrl_new(int fd, int node, int switchtype, pri_io_cb rd,
 	ctrl->q931_rxcount = 0;
 	ctrl->q931_txcount = 0;
 
+	ctrl->l2_persistence = pri_l2_persistence_option_default(ctrl);
 	ctrl->display_flags.send = pri_display_options_send_default(ctrl);
 	ctrl->display_flags.receive = pri_display_options_receive_default(ctrl);
 	switch (switchtype) {
@@ -951,7 +978,7 @@ void pri_copy_party_name_to_q931(struct q931_party_name *q931_name, const struct
 	q931_party_name_init(q931_name);
 	if (pri_name->valid) {
 		q931_name->valid = 1;
-		q931_name->presentation = pri_name->presentation;
+		q931_name->presentation = pri_name->presentation & PRI_PRES_RESTRICTION;
 		q931_name->char_set = pri_name->char_set;
 		libpri_copy_string(q931_name->str, pri_name->str, sizeof(q931_name->str));
 	}
@@ -970,7 +997,8 @@ void pri_copy_party_number_to_q931(struct q931_party_number *q931_number, const
 	q931_party_number_init(q931_number);
 	if (pri_number->valid) {
 		q931_number->valid = 1;
-		q931_number->presentation = pri_number->presentation;
+		q931_number->presentation = pri_number->presentation
+			& (PRI_PRES_RESTRICTION | PRI_PRES_NUMBER_TYPE);
 		q931_number->plan = pri_number->plan;
 		libpri_copy_string(q931_number->str, pri_number->str, sizeof(q931_number->str));
 	}
@@ -1772,7 +1800,8 @@ char *pri_dump_info_str(struct pri *ctrl)
 			enum PRI_TIMERS_AND_COUNTERS tmr;
 
 			tmr = pri_timer[idx].number;
-			if (0 <= ctrl->timers[tmr]) {
+			if (0 <= ctrl->timers[tmr]
+				|| tmr == PRI_TIMER_T316) {
 				used = pri_snprintf(buf, used, buf_size, "  %s: %d\n",
 					pri_timer[idx].name, ctrl->timers[tmr]);
 			}
@@ -1940,13 +1969,14 @@ int pri_sr_set_caller(struct pri_sr *sr, char *caller, char *callername, int cal
 	q931_party_id_init(&sr->caller);
 	if (caller) {
 		sr->caller.number.valid = 1;
-		sr->caller.number.presentation = callerpres;
+		sr->caller.number.presentation = callerpres
+			& (PRI_PRES_RESTRICTION | PRI_PRES_NUMBER_TYPE);
 		sr->caller.number.plan = callerplan;
 		libpri_copy_string(sr->caller.number.str, caller, sizeof(sr->caller.number.str));
 
 		if (callername) {
 			sr->caller.name.valid = 1;
-			sr->caller.name.presentation = callerpres;
+			sr->caller.name.presentation = callerpres & PRI_PRES_RESTRICTION;
 			sr->caller.name.char_set = PRI_CHAR_SET_ISO8859_1;
 			libpri_copy_string(sr->caller.name.str, callername,
 				sizeof(sr->caller.name.str));
@@ -1970,7 +2000,8 @@ int pri_sr_set_redirecting(struct pri_sr *sr, char *num, int plan, int pres, int
 	q931_party_redirecting_init(&sr->redirecting);
 	if (num && num[0]) {
 		sr->redirecting.from.number.valid = 1;
-		sr->redirecting.from.number.presentation = pres;
+		sr->redirecting.from.number.presentation = pres
+			& (PRI_PRES_RESTRICTION | PRI_PRES_NUMBER_TYPE);
 		sr->redirecting.from.number.plan = plan;
 		libpri_copy_string(sr->redirecting.from.number.str, num,
 			sizeof(sr->redirecting.from.number.str));
@@ -2165,6 +2196,27 @@ void pri_cc_retain_signaling_rsp(struct pri *ctrl, int signaling_retention)
 	}
 }
 
+void pri_persistent_layer2_option(struct pri *ctrl, enum pri_layer2_persistence option)
+{
+	if (!ctrl) {
+		return;
+	}
+	if (PTMP_MODE(ctrl)) {
+		switch (option) {
+		case PRI_L2_PERSISTENCE_DEFAULT:
+			ctrl->l2_persistence = pri_l2_persistence_option_default(ctrl);
+			break;
+		case PRI_L2_PERSISTENCE_KEEP_UP:
+		case PRI_L2_PERSISTENCE_LEAVE_DOWN:
+			ctrl->l2_persistence = option;
+			break;
+		}
+		if (ctrl->l2_persistence == PRI_L2_PERSISTENCE_KEEP_UP) {
+			q921_bring_layer2_up(ctrl);
+		}
+	}
+}
+
 void pri_display_options_send(struct pri *ctrl, unsigned long flags)
 {
 	if (!ctrl) {
