@@ -120,6 +120,8 @@ struct pri {
 
 	/*! Layer 2 link control for D channel. */
 	struct q921_link link;
+	/*! Layer 2 persistence option. */
+	enum pri_layer2_persistence l2_persistence;
 	/*! T201 TEI Identity Check timer. */
 	int t201_timer;
 	/*! Number of times T201 has expired. */
@@ -623,7 +625,9 @@ struct q931_call {
 	/*! Display text ie contents. */
 	struct {
 		/*! Display ie text.  NULL if not present or consumed as remote name. */
-		const char *text;
+		const unsigned char *text;
+		/*! Full IE code of received display text */
+		int full_ie;
 		/*! Length of display text. */
 		unsigned char length;
 		/*!
@@ -645,6 +649,29 @@ struct q931_call {
 
 	/* AOC charge requesting on Setup */
 	int aoc_charging_request;
+
+	unsigned int slotmap_size:1;/* TRUE if the slotmap is E1 (32 bits). */
+
+	/*! Control the RESTART reception to the upper layer. */
+	struct {
+		/*! Timer ID of RESTART notification events to upper layer. */
+		int timer;
+		/*! Current RESTART notification index. */
+		int idx;
+		/*! Number of channels in the channel ID list. */
+		int count;
+		/*! Channel ID list */
+		char chan_no[32];
+	} restart;
+	/*! Control the RESTART retransmissions. */
+	struct {
+		/*! T316 RESTART retransmit timer. */
+		int t316_timer;
+		/*! Number of times remaining that RESTART can be transmitted. */
+		int remain;
+		/*! Encoded RESTART channel id. */
+		int channel;
+	} restart_tx;
 };
 
 enum CC_STATES {
