@@ -82,6 +82,9 @@ typedef struct q931_ie {
 
 /* Q.931 / National ISDN Message Types */
 
+/*! Send this facility APDU on the next message to go out. */
+#define Q931_ANY_MESSAGE			-1
+
 /* Call Establishment Messages */
 #define Q931_ALERTING 				0x01
 #define Q931_CALL_PROCEEDING		0x02
@@ -106,6 +109,7 @@ typedef struct q931_ie {
 #define Q931_CONGESTION_CONTROL		0x79
 #define Q931_INFORMATION			0x7b
 #define Q931_FACILITY				0x62
+#define Q931_REGISTER				0x64	/* Q.932 */
 #define Q931_NOTIFY					0x6e
 
 /* Call Management Messages */
@@ -439,6 +443,8 @@ enum Q931_RANKED_CALL_STATE {
 	Q931_RANKED_CALL_STATE_CONNECT,
 	/*! Call is in some non-call establishment state (likely disconnecting). */
 	Q931_RANKED_CALL_STATE_OTHER,
+	/*! Master call is aborting. */
+	Q931_RANKED_CALL_STATE_ABORT,
 };
 
 /* EuroISDN  */
@@ -446,13 +452,11 @@ enum Q931_RANKED_CALL_STATE {
 
 extern int maintenance_service(struct pri *pri, int span, int channel, int changestatus);
 
-extern int maintenance_service_ack(struct pri *pri, q931_call *call);
-
 
 /* Q.SIG specific */
 #define QSIG_IE_TRANSIT_COUNT		0x31
 
-extern int q931_receive(struct pri *pri, int tei, q931_h *h, int len);
+int q931_receive(struct q921_link *link, q931_h *h, int len);
 
 extern int q931_alerting(struct pri *pri, q931_call *call, int channel, int info);
 
@@ -471,6 +475,7 @@ extern int q931_information(struct pri *pri, q931_call *call, char digit);
 extern int q931_keypad_facility(struct pri *pri, q931_call *call, const char *digits);
 
 extern int q931_connect(struct pri *pri, q931_call *call, int channel, int nonisdn);
+int q931_connect_acknowledge(struct pri *ctrl, q931_call *call, int channel);
 
 extern int q931_release(struct pri *pri, q931_call *call, int cause);
 
@@ -486,15 +491,25 @@ extern int q931_call_getcrv(struct pri *pri, q931_call *call, int *callmode);
 
 extern int q931_call_setcrv(struct pri *pri, q931_call *call, int crv, int callmode);
 
-struct q931_call *q931_find_call(struct pri *ctrl, int cr);
-struct q931_call *q931_new_call(struct pri *pri);
+struct q931_call *q931_new_call(struct pri *ctrl);
 
 extern int q931_setup(struct pri *pri, q931_call *c, struct pri_sr *req);
+
+int q931_register(struct pri *ctrl, q931_call *call);
+
 void q931_dump(struct pri *ctrl, int tei, q931_h *h, int len, int txrx);
 
 void q931_destroycall(struct pri *pri, q931_call *c);
 
-extern void q931_dl_indication(struct pri *pri, int event);
+enum Q931_DL_EVENT {
+	Q931_DL_EVENT_NONE,
+	Q931_DL_EVENT_DL_ESTABLISH_IND,
+	Q931_DL_EVENT_DL_ESTABLISH_CONFIRM,
+	Q931_DL_EVENT_DL_RELEASE_IND,
+	Q931_DL_EVENT_DL_RELEASE_CONFIRM,
+	Q931_DL_EVENT_TEI_REMOVAL,
+};
+void q931_dl_event(struct q921_link *link, enum Q931_DL_EVENT event);
 
 int q931_send_hold(struct pri *ctrl, struct q931_call *call);
 int q931_send_hold_ack(struct pri *ctrl, struct q931_call *call);
