@@ -186,7 +186,7 @@ static int q921_transmit(struct pri *ctrl, q921_h *h, int len)
 	ctrl->q921_txcount++;
 	/* Just send it raw */
 	if (ctrl->debug & (PRI_DEBUG_Q921_DUMP | PRI_DEBUG_Q921_RAW))
-		q921_dump(ctrl, h, len, ctrl->debug & PRI_DEBUG_Q921_RAW, 1);
+		q921_dump(ctrl, h, len, ctrl->debug, 1);
 	/* Write an extra two bytes for the FCS */
 	res = ctrl->write_func ? ctrl->write_func(ctrl, h, len + 2) : 0;
 	if (res != (len + 2)) {
@@ -235,7 +235,10 @@ static void t202_expire(void *vlink)
 	link->t202_timer =
 		pri_schedule_event(ctrl, ctrl->timers[PRI_TIMER_T202], t202_expire, link);
 
-	++link->n202_counter;
+	if (ctrl->l2_persistence != PRI_L2_PERSISTENCE_KEEP_UP) {
+		/* Only try to get a TEI for N202 times if layer 2 is not persistent. */
+		++link->n202_counter;
+	}
 	if (!link->t202_timer || link->n202_counter > ctrl->timers[PRI_TIMER_N202]) {
 		if (!link->t202_timer) {
 			pri_error(ctrl, "Could not start T202 timer.");
@@ -530,6 +533,49 @@ static void stop_t200(struct q921_link *link)
 	}
 }
 
+/*!
+ * \internal
+ * \brief Initiate bringing up layer 2 link.
+ *
+ * \param link Layer 2 link to bring up.
+ *
+ * \return Nothing
+ */
+static void kick_start_link(struct q921_link *link)
+{
+	struct pri *ctrl;
+
+	ctrl = link->ctrl;
+
+	switch (link->state) {
+	case Q921_TEI_UNASSIGNED:
+		if (ctrl->debug & PRI_DEBUG_Q921_STATE) {
+			pri_message(ctrl, "Kick starting link from no TEI.\n");
+		}
+		q921_setstate(link, Q921_ESTABLISH_AWAITING_TEI);
+		q921_tei_request(link);
+		break;
+	case Q921_ASSIGN_AWAITING_TEI:
+		if (ctrl->debug & PRI_DEBUG_Q921_STATE) {
+			pri_message(ctrl, "Kick starting link when get TEI.\n");
+		}
+		q921_setstate(link, Q921_ESTABLISH_AWAITING_TEI);
+		break;
+	case Q921_TEI_ASSIGNED:
+		if (ctrl->debug & PRI_DEBUG_Q921_STATE) {
+			pri_message(ctrl, "SAPI/TEI=%d/%d Kick starting link\n", link->sapi,
+				link->tei);
+		}
+		q921_discard_iqueue(link);
+		q921_establish_data_link(link);
+		link->l3_initiated = 1;
+		q921_setstate(link, Q921_AWAITING_ESTABLISHMENT);
+		break;
+	default:
+		break;
+	}
+}
+
 static void restart_timer_expire(void *vlink)
 {
 	struct q921_link *link = vlink;
@@ -537,19 +583,12 @@ static void restart_timer_expire(void *vlink)
 
 	ctrl = link->ctrl;
 
-	if (ctrl->debug & PRI_DEBUG_Q921_STATE) {
-		pri_message(ctrl, "SAPI/TEI=%d/%d Kick starting link\n", link->sapi, link->tei);
-	}
-
 	link->restart_timer = 0;
 
 	switch (link->state) {
 	case Q921_TEI_ASSIGNED:
 		/* Try to bring layer 2 up. */
-		q921_discard_iqueue(link);
-		q921_establish_data_link(link);
-		link->l3_initiated = 1;
-		q921_setstate(link, Q921_AWAITING_ESTABLISHMENT);
+		kick_start_link(link);
 		break;
 	default:
 		/* Looks like someone forgot to stop the restart timer. */
@@ -568,6 +607,7 @@ static void restart_timer_stop(struct q921_link *link)
 	link->restart_timer = 0;
 }
 
+/*! \note Only call on the transition to state Q921_TEI_ASSIGNED or already there. */
 static void restart_timer_start(struct q921_link *link)
 {
 	struct pri *ctrl;
@@ -583,21 +623,27 @@ static void restart_timer_start(struct q921_link *link)
 		pri_schedule_event(ctrl, ctrl->timers[PRI_TIMER_T200], restart_timer_expire, link);
 }
 
-static pri_event *q921_ptp_delay_restart(struct q921_link *link)
+/*! \note Only call on the transition to state Q921_TEI_ASSIGNED or already there. */
+static pri_event *q921_check_delay_restart(struct q921_link *link)
 {
 	pri_event *ev;
 	struct pri *ctrl;
 
 	ctrl = link->ctrl;
 
-	if (PTP_MODE(ctrl)) {
+	if (ctrl->l2_persistence == PRI_L2_PERSISTENCE_KEEP_UP) {
 		/*
+		 * For PTP links:
 		 * This is where we act a bit like L3 instead of L2, since we've
 		 * got an L3 that depends on us keeping L2 automatically alive
-		 * and happy for PTP links.
+		 * and happy.
+		 *
+		 * For PTMP links:
+		 * We can optionally keep L2 automatically alive and happy.
 		 */
 		restart_timer_start(link);
-
+	}
+	if (PTP_MODE(ctrl)) {
 		switch (link->state) {
 		case Q921_MULTI_FRAME_ESTABLISHED:
 		case Q921_TIMER_RECOVERY:
@@ -616,6 +662,31 @@ static pri_event *q921_ptp_delay_restart(struct q921_link *link)
 	return ev;
 }
 
+/*!
+ * \brief Bring all layer 2 links up.
+ *
+ * \param ctrl D channel controller.
+ *
+ * \return Nothing
+ */
+void q921_bring_layer2_up(struct pri *ctrl)
+{
+	struct q921_link *link;
+
+	if (PTMP_MODE(ctrl)) {
+		/* Don't start with the broadcast link. */
+		link = ctrl->link.next;
+	} else {
+		link = &ctrl->link;
+	}
+	for (; link; link = link->next) {
+		if (!link->restart_timer) {
+			/* A restart on the link is not already in the works. */
+			kick_start_link(link);
+		}
+	}
+}
+
 /* This is the equivalent of the I-Frame queued up path in Figure B.7 in MULTI_FRAME_ESTABLISHED */
 static int q921_send_queued_iframes(struct q921_link *link)
 {
@@ -859,7 +930,7 @@ static void t200_expire(void *vlink)
 			q921_send_sabme(link);
 			start_t200(link);
 		} else {
-			q921_ptp_delay_restart(link);
+			q921_check_delay_restart(link);
 			q921_discard_iqueue(link);
 			q921_mdl_error(link, 'G');
 			q921_setstate(link, Q921_TEI_ASSIGNED);
@@ -873,7 +944,7 @@ static void t200_expire(void *vlink)
 			q921_send_disc(link, 1);
 			start_t200(link);
 		} else {
-			q921_ptp_delay_restart(link);
+			q921_check_delay_restart(link);
 			q921_mdl_error(link, 'H');
 			/* DL-RELEASE confirm */
 			q931_dl_event(link, Q931_DL_EVENT_DL_RELEASE_CONFIRM);
@@ -1116,7 +1187,7 @@ static void q921_dump_iqueue_info(struct q921_link *link)
 
 static void q921_dump_pri_by_h(struct pri *ctrl, char direction_tag, q921_h *h);
 
-void q921_dump(struct pri *ctrl, q921_h *h, int len, int showraw, int txrx)
+void q921_dump(struct pri *ctrl, q921_h *h, int len, int debugflags, int txrx)
 {
 	int x;
 	const char *type;
@@ -1125,9 +1196,11 @@ void q921_dump(struct pri *ctrl, q921_h *h, int len, int showraw, int txrx)
 	direction_tag = txrx ? '>' : '<';
 
 	pri_message(ctrl, "\n");
-	q921_dump_pri_by_h(ctrl, direction_tag, h);
+	if (debugflags & PRI_DEBUG_Q921_DUMP) {
+		q921_dump_pri_by_h(ctrl, direction_tag, h);
+	}
 
-	if (showraw) {
+	if (debugflags & PRI_DEBUG_Q921_RAW) {
 		char *buf = malloc(len * 3 + 1);
 		int buflen = 0;
 		if (buf) {
@@ -1138,132 +1211,134 @@ void q921_dump(struct pri *ctrl, q921_h *h, int len, int showraw, int txrx)
 		}
 	}
 
-	switch (h->h.data[0] & Q921_FRAMETYPE_MASK) {
-	case 0:
-	case 2:
-		pri_message(ctrl, "%c Informational frame:\n", direction_tag);
-		break;
-	case 1:
-		pri_message(ctrl, "%c Supervisory frame:\n", direction_tag);
-		break;
-	case 3:
-		pri_message(ctrl, "%c Unnumbered frame:\n", direction_tag);
-		break;
-	}
-	
-	pri_message(ctrl, "%c SAPI: %02d  C/R: %d EA: %d\n",
-		direction_tag,
-		h->h.sapi, 
-		h->h.c_r,
-		h->h.ea1);
-	pri_message(ctrl, "%c  TEI: %03d        EA: %d\n", 
-		direction_tag,
-		h->h.tei,
-		h->h.ea2);
-
-	switch (h->h.data[0] & Q921_FRAMETYPE_MASK) {
-	case 0:
-	case 2:
-		/* Informational frame */
-		pri_message(ctrl, "%c N(S): %03d   0: %d\n",
-			direction_tag,
-			h->i.n_s,
-			h->i.ft);
-		pri_message(ctrl, "%c N(R): %03d   P: %d\n",
-			direction_tag,
-			h->i.n_r,
-			h->i.p_f);
-		pri_message(ctrl, "%c %d bytes of data\n",
-			direction_tag,
-			len - 4);
-		break;
-	case 1:
-		/* Supervisory frame */
-		type = "???";
-		switch (h->s.ss) {
+	if (debugflags & PRI_DEBUG_Q921_DUMP) {
+		switch (h->h.data[0] & Q921_FRAMETYPE_MASK) {
 		case 0:
-			type = "RR (receive ready)";
+		case 2:
+			pri_message(ctrl, "%c Informational frame:\n", direction_tag);
 			break;
 		case 1:
-			type = "RNR (receive not ready)";
+			pri_message(ctrl, "%c Supervisory frame:\n", direction_tag);
 			break;
-		case 2:
-			type = "REJ (reject)";
+		case 3:
+			pri_message(ctrl, "%c Unnumbered frame:\n", direction_tag);
 			break;
 		}
-		pri_message(ctrl, "%c Zero: %d     S: %d 01: %d  [ %s ]\n",
+		
+		pri_message(ctrl, "%c SAPI: %02d  C/R: %d EA: %d\n",
 			direction_tag,
-			h->s.x0,
-			h->s.ss,
-			h->s.ft,
-			type);
-		pri_message(ctrl, "%c N(R): %03d P/F: %d\n",
+			h->h.sapi, 
+			h->h.c_r,
+			h->h.ea1);
+		pri_message(ctrl, "%c  TEI: %03d        EA: %d\n", 
 			direction_tag,
-			h->s.n_r,
-			h->s.p_f);
-		pri_message(ctrl, "%c %d bytes of data\n",
-			direction_tag,
-			len - 4);
-		break;
-	case 3:		
-		/* Unnumbered frame */
-		type = "???";
-		if (h->u.ft == 3) {
-			switch (h->u.m3) {
+			h->h.tei,
+			h->h.ea2);
+	
+		switch (h->h.data[0] & Q921_FRAMETYPE_MASK) {
+		case 0:
+		case 2:
+			/* Informational frame */
+			pri_message(ctrl, "%c N(S): %03d   0: %d\n",
+				direction_tag,
+				h->i.n_s,
+				h->i.ft);
+			pri_message(ctrl, "%c N(R): %03d   P: %d\n",
+				direction_tag,
+				h->i.n_r,
+				h->i.p_f);
+			pri_message(ctrl, "%c %d bytes of data\n",
+				direction_tag,
+				len - 4);
+			break;
+		case 1:
+			/* Supervisory frame */
+			type = "???";
+			switch (h->s.ss) {
 			case 0:
-				if (h->u.m2 == 3)
-					type = "DM (disconnect mode)";
-				else if (h->u.m2 == 0)
-					type = "UI (unnumbered information)";
+				type = "RR (receive ready)";
+				break;
+			case 1:
+				type = "RNR (receive not ready)";
 				break;
 			case 2:
-				if (h->u.m2 == 0)
-					type = "DISC (disconnect)";
-				break;
-			case 3:
-				if (h->u.m2 == 3)
-					type = "SABME (set asynchronous balanced mode extended)";
-				else if (h->u.m2 == 0)
-					type = "UA (unnumbered acknowledgement)";
-				break;
-			case 4:
-				if (h->u.m2 == 1)
-					type = "FRMR (frame reject)";
-				break;
-			case 5:
-				if (h->u.m2 == 3)
-					type = "XID (exchange identification note)";
-				break;
-			default:
+				type = "REJ (reject)";
 				break;
 			}
+			pri_message(ctrl, "%c Zero: %d     S: %d 01: %d  [ %s ]\n",
+				direction_tag,
+				h->s.x0,
+				h->s.ss,
+				h->s.ft,
+				type);
+			pri_message(ctrl, "%c N(R): %03d P/F: %d\n",
+				direction_tag,
+				h->s.n_r,
+				h->s.p_f);
+			pri_message(ctrl, "%c %d bytes of data\n",
+				direction_tag,
+				len - 4);
+			break;
+		case 3:		
+			/* Unnumbered frame */
+			type = "???";
+			if (h->u.ft == 3) {
+				switch (h->u.m3) {
+				case 0:
+					if (h->u.m2 == 3)
+						type = "DM (disconnect mode)";
+					else if (h->u.m2 == 0)
+						type = "UI (unnumbered information)";
+					break;
+				case 2:
+					if (h->u.m2 == 0)
+						type = "DISC (disconnect)";
+					break;
+				case 3:
+					if (h->u.m2 == 3)
+						type = "SABME (set asynchronous balanced mode extended)";
+					else if (h->u.m2 == 0)
+						type = "UA (unnumbered acknowledgement)";
+					break;
+				case 4:
+					if (h->u.m2 == 1)
+						type = "FRMR (frame reject)";
+					break;
+				case 5:
+					if (h->u.m2 == 3)
+						type = "XID (exchange identification note)";
+					break;
+				default:
+					break;
+				}
+			}
+			pri_message(ctrl, "%c   M3: %d   P/F: %d M2: %d 11: %d  [ %s ]\n",
+				direction_tag,
+				h->u.m3,
+				h->u.p_f,
+				h->u.m2,
+				h->u.ft,
+				type);
+			pri_message(ctrl, "%c %d bytes of data\n",
+				direction_tag,
+				len - 3);
+			break;
 		}
-		pri_message(ctrl, "%c   M3: %d   P/F: %d M2: %d 11: %d  [ %s ]\n",
-			direction_tag,
-			h->u.m3,
-			h->u.p_f,
-			h->u.m2,
-			h->u.ft,
-			type);
-		pri_message(ctrl, "%c %d bytes of data\n",
-			direction_tag,
-			len - 3);
-		break;
-	}
-
-	if ((h->u.ft == 3) && (h->u.m3 == 0) && (h->u.m2 == 0) && (h->u.data[0] == 0x0f)) {
-		int ri;
-		u_int8_t *action;
-
-		/* TEI management related */
-		type = q921_tei_mgmt2str(h->u.data[3]);
-		pri_message(ctrl, "%c MDL Message: %d(%s)\n", direction_tag, h->u.data[3], type);
-		ri = (h->u.data[1] << 8) | h->u.data[2];
-		pri_message(ctrl, "%c Ri: %d\n", direction_tag, ri);
-		action = &h->u.data[4];
-		for (x = len - (action - (u_int8_t *) h); 0 < x; --x, ++action) {
-			pri_message(ctrl, "%c Ai: %d E:%d\n",
-				direction_tag, (*action >> 1) & 0x7f, *action & 0x01);
+	
+		if ((h->u.ft == 3) && (h->u.m3 == 0) && (h->u.m2 == 0) && (h->u.data[0] == 0x0f)) {
+			int ri;
+			u_int8_t *action;
+	
+			/* TEI management related */
+			type = q921_tei_mgmt2str(h->u.data[3]);
+			pri_message(ctrl, "%c MDL Message: %d(%s)\n", direction_tag, h->u.data[3], type);
+			ri = (h->u.data[1] << 8) | h->u.data[2];
+			pri_message(ctrl, "%c Ri: %d\n", direction_tag, ri);
+			action = &h->u.data[4];
+			for (x = len - (action - (u_int8_t *) h); 0 < x; --x, ++action) {
+				pri_message(ctrl, "%c Ai: %d E:%d\n",
+					direction_tag, (*action >> 1) & 0x7f, *action & 0x01);
+			}
 		}
 	}
 }
@@ -1409,8 +1484,20 @@ static pri_event *q921_receive_MDL(struct pri *ctrl, q921_u *h, int len)
 	int tei;
 
 	if (!BRI_NT_PTMP(ctrl) && !BRI_TE_PTMP(ctrl)) {
-		return pri_mkerror(ctrl,
-			"Received MDL/TEI managemement message, but configured for mode other than PTMP!\n");
+		/*
+		 * Some telco switches send out MDL messages even though they
+		 * are configured for PTP.  Usually they are checking for
+		 * assigned TEI's.
+		 */
+		if (ctrl->debug & PRI_DEBUG_Q921_STATE) {
+			/*
+			 * Send out this message in debug modes since it is possible the
+			 * user has misconfigured their link for the wrong mode.
+			 */
+			pri_message(ctrl, "Not configured for PTMP.  Ignoring MDL message: %d(%s)\n",
+				h->data[3], q921_tei_mgmt2str(h->data[3]));
+		}
+		return NULL;
 	}
 
 	if (ctrl->debug & PRI_DEBUG_Q921_STATE) {
@@ -1471,12 +1558,13 @@ static pri_event *q921_receive_MDL(struct pri *ctrl, q921_u *h, int len)
 		if (ctrl->debug & PRI_DEBUG_Q921_STATE) {
 			pri_message(ctrl, "Allocating new TEI %d\n", tei);
 		}
-		sub->next = pri_link_new(ctrl, Q921_SAPI_CALL_CTRL, tei);
-		if (!sub->next) {
+		link = pri_link_new(ctrl, Q921_SAPI_CALL_CTRL, tei);
+		if (!link) {
 			pri_error(ctrl, "Unable to allocate layer 2 link for new TEI %d\n", tei);
 			return NULL;
 		}
-		q921_setstate(sub->next, Q921_TEI_ASSIGNED);
+		sub->next = link;
+		q921_setstate(link, Q921_TEI_ASSIGNED);
 		q921_send_tei(ctrl, Q921_TEI_IDENTITY_ASSIGNED, ri, tei, 1);
 
 		count = 0;
@@ -1493,6 +1581,15 @@ static pri_event *q921_receive_MDL(struct pri *ctrl, q921_u *h, int len)
 			}
 			q921_tei_check(ctrl);
 		}
+
+		if (ctrl->l2_persistence == PRI_L2_PERSISTENCE_KEEP_UP) {
+			/*
+			 * Layer 2 is persistent so give the peer some time to setup
+			 * it's new TEI and bring the link up itself before we bring the
+			 * link up.
+			 */
+			restart_timer_start(link);
+		}
 		break;
 	case Q921_TEI_IDENTITY_CHECK_RESPONSE:
 		if (!BRI_NT_PTMP(ctrl)) {
@@ -1562,7 +1659,12 @@ static pri_event *q921_receive_MDL(struct pri *ctrl, q921_u *h, int len)
 		
 		switch (link->state) {
 		case Q921_TEI_UNASSIGNED:
-			/* We do not have a TEI and we are not asking for one. */
+			/*
+			 * We do not have a TEI and we are not currently asking for one.
+			 * Start asking for one.
+			 */
+			q921_setstate(link, Q921_ASSIGN_AWAITING_TEI);
+			q921_tei_request(link);
 			return NULL;
 		case Q921_ASSIGN_AWAITING_TEI:
 		case Q921_ESTABLISH_AWAITING_TEI:
@@ -1604,9 +1706,12 @@ static pri_event *q921_receive_MDL(struct pri *ctrl, q921_u *h, int len)
 		switch (link->state) {
 		case Q921_ASSIGN_AWAITING_TEI:
 			q921_setstate(link, Q921_TEI_ASSIGNED);
-			ctrl->ev.gen.e = PRI_EVENT_DCHAN_UP;
-			res = &ctrl->ev;
-			break;
+			if (ctrl->l2_persistence != PRI_L2_PERSISTENCE_KEEP_UP) {
+				ctrl->ev.gen.e = PRI_EVENT_DCHAN_UP;
+				res = &ctrl->ev;
+				break;
+			}
+			/* Fall through: Layer 2 is persistent so bring it up. */
 		case Q921_ESTABLISH_AWAITING_TEI:
 			q921_establish_data_link(link);
 			link->l3_initiated = 1;
@@ -1765,7 +1870,7 @@ static pri_event *q921_disc_rx(struct q921_link *link, q921_h *h)
 		break;
 	case Q921_MULTI_FRAME_ESTABLISHED:
 	case Q921_TIMER_RECOVERY:
-		res = q921_ptp_delay_restart(link);
+		res = q921_check_delay_restart(link);
 		q921_discard_iqueue(link);
 		q921_send_ua(link, h->u.p_f);
 		/* DL-RELEASE indication */
@@ -2177,7 +2282,7 @@ static pri_event *q921_ua_rx(struct q921_link *link, q921_h *h)
 		if (!h->u.p_f) {
 			q921_mdl_error(link, 'D');
 		} else {
-			res = q921_ptp_delay_restart(link);
+			res = q921_check_delay_restart(link);
 			/* DL-RELEASE confirm */
 			q931_dl_event(link, Q931_DL_EVENT_DL_RELEASE_CONFIRM);
 			stop_t200(link);
@@ -2615,7 +2720,7 @@ static pri_event *q921_dm_rx(struct q921_link *link, q921_h *h)
 		if (!h->u.p_f)
 			break;
 
-		res = q921_ptp_delay_restart(link);
+		res = q921_check_delay_restart(link);
 		q921_discard_iqueue(link);
 		/* DL-RELEASE indication */
 		q931_dl_event(link, Q931_DL_EVENT_DL_RELEASE_IND);
@@ -2625,7 +2730,7 @@ static pri_event *q921_dm_rx(struct q921_link *link, q921_h *h)
 	case Q921_AWAITING_RELEASE:
 		if (!h->u.p_f)
 			break;
-		res = q921_ptp_delay_restart(link);
+		res = q921_check_delay_restart(link);
 		/* DL-RELEASE confirm */
 		q931_dl_event(link, Q931_DL_EVENT_DL_RELEASE_CONFIRM);
 		stop_t200(link);
@@ -2917,7 +3022,7 @@ static pri_event *__q921_receive(struct pri *ctrl, q921_h *h, int len)
 	len -= 2;
 	
 	if (ctrl->debug & (PRI_DEBUG_Q921_DUMP | PRI_DEBUG_Q921_RAW)) {
-		q921_dump(ctrl, h, len, ctrl->debug & PRI_DEBUG_Q921_RAW, 0);
+		q921_dump(ctrl, h, len, ctrl->debug, 0);
 	}
 
 	/* Check some reject conditions -- Start by rejecting improper ea's */
